"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    MIDIMessageEvent: ()=>_midi.MIDIMessageEvent,
    SchedulerTargetProcessor: ()=>SchedulerTargetProcessor
});
const _aliceBob = require("alice-bob");
const _messageQueue = require("./message-queue");
const _midi = require("./midi");
const sortByFrame = (a, b)=>a.offsetFrame - b.offsetFrame;
const midiEvents = [];
const midiEventPool = Array.from({
    length: 128
}, ()=>new _midi.MIDIMessageEvent('midimessage', {
        data: new Uint8Array(3)
    }));
let midiEventPoolPtr = 0;
let SchedulerTargetProcessor = class SchedulerTargetProcessor extends AudioWorkletProcessor {
    midiQueue = new _messageQueue.MessageQueue();
    didError = false;
    constructor(){
        super();
        const [worklet] = new _aliceBob.Bob((data)=>this.port.postMessage(data), this).agents({
            debug: false
        });
        this.port.onmessage = ({ data  })=>worklet.receive(data);
    }
    async resetError() {
        this.didError = false;
    }
    async init(buffer) {
        this.midiQueue.buffer = buffer;
    }
    process(inputs, outputs, parameters) {
        if (this.didError) return true;
        // TODO: optimizations possible:
        //  reuse midiEvents
        //  events pool
        //  sort by hash?
        let message;
        let prevFrame = 0;
        let max = 16;
        midiEvents.splice(0);
        while((message = this.midiQueue.slice(4)) && max--){
            const event = midiEventPool[midiEventPoolPtr];
            midiEventPoolPtr = (midiEventPoolPtr + 1) % midiEventPool.length;
            event.data.set(message.subarray(1));
            event.receivedTime = message[0] || currentTime * 1000;
            event.receivedFrame = Math.floor(event.receivedTime * 0.001 * sampleRate);
            event.offsetFrame = event.receivedFrame - currentFrame;
            event.deltaFrame = Math.max(0, event.offsetFrame - prevFrame);
            prevFrame = event.offsetFrame;
            midiEvents.push(event);
        }
        if (midiEvents.length) {
            midiEvents.sort(sortByFrame);
            this.processMidiEvents(midiEvents);
        }
        try {
            return this.processWithMidi(inputs, outputs, parameters, midiEvents);
        } catch (error) {
            console.warn(error);
            this.didError = true;
            return true;
        }
    }
};

//# sourceMappingURL=scheduler-target-processor.js.map