"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    roundPrecision: ()=>roundPrecision,
    floorPrecision: ()=>floorPrecision,
    ceilPrecision: ()=>ceilPrecision,
    getEventsInRange: ()=>getEventsInRange
});
const _schedulerEvent = require("./scheduler-event");
function roundPrecision(num, precision) {
    const factor = Math.pow(10, precision.toString().split('.')[1].length);
    return Math.round(num * factor) / factor;
}
function floorPrecision(num, precision) {
    const factor = Math.pow(10, precision.toString().split('.')[1].length);
    return Math.floor(num * factor) / factor;
}
function ceilPrecision(num, precision) {
    const factor = Math.pow(10, precision.toString().split('.')[1].length);
    return Math.ceil(num * factor) / factor;
}
let prevTurn = -1;
const getEventsInRange = (turns, loop, loopStart, loopEnd, currentTime, playbackStartTime, quantumDurationTime, sampleTime)=>{
    let startTime;
    let endTime;
    let offsetTime = 0;
    let loopDuration = 0;
    const results = [];
    let needTurn = 0;
    let turn = 0;
    if (loop) {
        loopDuration = loopEnd - loopStart;
        offsetTime = currentTime - loopStart;
        turn = offsetTime / loopDuration | 0;
        needTurn = turn + 2;
        if (turn !== prevTurn) {
            prevTurn = turn;
        }
        startTime = offsetTime % loopDuration;
        if (startTime < 0) {
            return {
                needTurn,
                results
            };
        }
        endTime = startTime + quantumDurationTime;
        endTime = ceilPrecision(endTime, sampleTime);
        if (startTime < sampleTime) {
            startTime = floorPrecision(startTime, sampleTime);
        }
    } else {
        startTime = currentTime - playbackStartTime;
        endTime = startTime + quantumDurationTime;
    }
    const isPast = endTime < startTime;
    const offset = Math[isPast ? 'ceil' : 'floor'](startTime / loopDuration) * loopDuration + playbackStartTime + (offsetTime - startTime);
    for(let t = 0; t <= 1; t++){
        const events = turns.get(turn + t);
        if (events) for (const event of events){
            const eventTime = event.midiEvent.receivedTime * 0.001 // ms to seconds
             + loopDuration * t;
            if (eventTime > endTime) break;
            if (loop === _schedulerEvent.LoopKind.Live) {
                if (eventTime >= startTime) {
                    let receivedTime = eventTime + offset;
                    receivedTime *= 1000;
                    results.push([
                        receivedTime,
                        event
                    ]);
                }
            } else if (eventTime === startTime && startTime === endTime || (isPast ? eventTime >= loopStart && eventTime < endTime : eventTime >= startTime && eventTime < endTime)) {
                let receivedTime1;
                if (loop) {
                    receivedTime1 = eventTime + offset;
                } else {
                    receivedTime1 = eventTime + playbackStartTime;
                }
                receivedTime1 *= 1000;
                results.push([
                    receivedTime1,
                    event
                ]);
            }
        }
    }
    return {
        needTurn,
        results
    };
};

//# sourceMappingURL=util.js.map