/// <reference types="webmidi" />
import { ValuesOf } from 'everyday-types';
import { ImmSet } from 'immutable-map-set';
import { MessageQueue } from './message-queue';
import { SchedulerNode } from './scheduler-node';
import { SchedulerTargetNode } from './scheduler-target-node';
export declare type NoteEvent = [number, number, number, number];
export declare const LoopKind: {
    readonly Once: 0;
    readonly Loop: 1;
    readonly Live: 2;
};
export declare type LoopKind = ValuesOf<typeof LoopKind>;
export declare class SchedulerEvent {
    id: string;
    midiEvent: {
        data: Uint8Array;
        receivedTime: number;
    };
    constructor(data?: Partial<SchedulerEvent>);
    toJSON(): {
        id: string;
        midiEvent: {
            receivedTime: number;
            data: Uint8Array;
        };
    };
}
export declare class SchedulerTarget {
    id: string;
    midiQueue: MessageQueue;
    constructor(data?: Partial<SchedulerTarget>);
}
export declare class SchedulerEventGroup {
    id: string;
    targets: ImmSet<SchedulerTarget>;
    scheduler?: SchedulerNode;
    loopPoints: Float64Array;
    constructor(eventGroup?: Partial<SchedulerEventGroup>);
    toJSON(): {
        id: string;
        targets: ImmSet<SchedulerTarget>;
        loopPoints: Float64Array;
    };
    get loop(): LoopKind;
    set loop(value: LoopKind);
    get loopStart(): number;
    set loopStart(seconds: number);
    get loopEnd(): number;
    set loopEnd(seconds: number);
    onRequestNotes?: (turn: number) => void;
    setMidiEvents(turnEvents: WebMidi.MIDIMessageEvent[][], turn?: number, clear?: boolean): WebMidi.MIDIMessageEvent[][];
    setNotes(turnNotes: NoteEvent[][], turn?: number, clear?: boolean): WebMidi.MIDIMessageEvent[][];
}
export declare function getMidiEventsForNotes(notes: NoteEvent[]): WebMidi.MIDIMessageEvent[];
export declare class SchedulerEventGroupNode extends EventTarget {
    schedulerNode: SchedulerNode;
    eventGroup: SchedulerEventGroup;
    targetNodes: Set<SchedulerTargetNode>;
    onconnectchange: (ev: CustomEvent) => void;
    constructor(schedulerNode: SchedulerNode);
    destroy(): void;
    suspend(targetNode: SchedulerTargetNode): void;
    resume(targetNode: SchedulerTargetNode): void;
    connect(targetNode: SchedulerTargetNode): SchedulerTargetNode;
    disconnect(targetNode: SchedulerTargetNode): void;
}
